@@ -1,7 +1,7 @@
 pub mod client;
 pub mod error;
-pub mod handler;
 pub mod hostname;
 mod packet;
 pub mod server;
 mod state;
+pub mod stream;
