@@ -1,26 +1,35 @@
+use crate::error::BrokenPipeError;
 use crate::hostname::Hostname;
 
 use crc::{Crc, CRC_16_IBM_SDLC};
+use std::sync::{mpsc, mpsc::TryRecvError};
 
 // 8 bytes for hostname
 // 8 bytes for hostname
 // 1 byte for packet type
+// 1 byte for seq num
 // Up to 2 bytes for packet metadata (frame id or # of frames)
 // 2 bytes for checksum
-pub(crate) const PACKET_OVERHEAD_BYTES: usize = 8 + 8 + 1 + 2 + 2;
+pub(crate) const PACKET_OVERHEAD_BYTES: usize = 8 + 8 + 1 + 1 + 2 + 2;
 
 const X25: Crc<u16> = Crc::<u16>::new(&CRC_16_IBM_SDLC);
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone)]
 pub(crate) struct Packet {
 	pub from: Hostname,
 	pub to: Hostname,
+	pub seq_num: u8,
 	pub data: PacketData,
 }
 
 impl Packet {
-	pub fn new(from: Hostname, to: Hostname, data: PacketData) -> Self {
-		Self { from, to, data }
+	pub fn new(from: Hostname, to: Hostname, seq_num: u8, data: PacketData) -> Self {
+		Self {
+			from,
+			to,
+			data,
+			seq_num,
+		}
 	}
 
 	pub fn to_bytes(&self, buf: &mut Vec<u8>) {
@@ -28,6 +37,7 @@ impl Packet {
 
 		self.from.to_bytes(buf);
 		self.to.to_bytes(buf);
+		buf.push(self.seq_num);
 		self.data.to_bytes(buf);
 
 		// Add checksum to tail of buffer
@@ -36,7 +46,7 @@ impl Packet {
 	}
 
 	pub fn from_bytes(buf: &[u8]) -> Option<Self> {
-		if buf.len() < 18 {
+		if buf.len() < 19 {
 			return None;
 		}
 
@@ -50,9 +60,15 @@ impl Packet {
 
 		let from = Hostname::from_bytes(&buf[0..8])?;
 		let to = Hostname::from_bytes(&buf[8..16])?;
-		let data = PacketData::from_bytes(&buf[16..])?;
+		let seq_num = buf[16];
+		let data = PacketData::from_bytes(&buf[17..])?;
 
-		Some(Self { from, to, data })
+		Some(Self {
+			from,
+			to,
+			seq_num,
+			data,
+		})
 	}
 }
 
@@ -67,10 +83,10 @@ const DATA_START_ACK: u8 = 7;
 const DATA_MID: u8 = 8;
 const DATA_RESEND: u8 = 9;
 
-#[derive(PartialEq, Eq, Debug)]
+#[derive(PartialEq, Eq, Debug, Clone)]
 pub(crate) enum PacketData {
-	Ping(u8),
-	Pong(u8),
+	Ping,
+	Pong,
 
 	Connect,
 	ConnectAck,
@@ -91,14 +107,12 @@ pub(crate) enum PacketData {
 impl PacketData {
 	pub fn to_bytes(&self, buf: &mut Vec<u8>) {
 		match self {
-			Self::Ping(id) => {
+			Self::Ping => {
 				buf.push(PING);
-				buf.push(*id);
 			}
 
-			Self::Pong(id) => {
+			Self::Pong => {
 				buf.push(PONG);
-				buf.push(*id);
 			}
 
 			Self::Connect => buf.push(CONNECT),
@@ -141,16 +155,16 @@ impl PacketData {
 
 		match id {
 			PING => {
-				if buf.len() == 1 {
-					Some(Self::Ping(buf[0]))
+				if buf.is_empty() {
+					Some(Self::Ping)
 				} else {
 					None
 				}
 			}
 
 			PONG => {
-				if buf.len() == 1 {
-					Some(Self::Pong(buf[0]))
+				if buf.is_empty() {
+					Some(Self::Pong)
 				} else {
 					None
 				}
@@ -246,6 +260,24 @@ fn u8s_to_u16(i: &[u8]) -> u16 {
 	i[0] as u16 + ((i[1] as u16) << 8)
 }
 
+pub(crate) fn try_read_packet(
+	packet_receiver: &mpsc::Receiver<Packet>,
+) -> Result<Option<Packet>, BrokenPipeError> {
+	match packet_receiver.try_recv() {
+		Ok(packet) => Ok(Some(packet)),
+		Err(TryRecvError::Empty) => Ok(None),
+		Err(TryRecvError::Disconnected) => Err(BrokenPipeError),
+	}
+}
+
+pub(crate) fn is_valid_seq_num(cur: u8, new: u8) -> bool {
+	if new > cur {
+		true
+	} else {
+		(255 - cur + new) < 100
+	}
+}
+
 #[cfg(test)]
 mod tests {
 	use super::*;
@@ -255,7 +287,8 @@ mod tests {
 		let packet = Packet::new(
 			Hostname::new("N0CALL-0").unwrap(),
 			Hostname::new("ABCDEFG-243").unwrap(),
-			PacketData::Ping(37),
+			37,
+			PacketData::Ping,
 		);
 
 		let mut buf = vec![];
@@ -270,6 +303,7 @@ mod tests {
 		let packet = Packet::new(
 			Hostname::new("N0CALL-0").unwrap(),
 			Hostname::new("ABCDEFG-243").unwrap(),
+			1,
 			PacketData::DataStart(
 				5,
 				vec![
@@ -290,6 +324,7 @@ mod tests {
 		let packet = Packet::new(
 			Hostname::new("N0CALL-0").unwrap(),
 			Hostname::new("ABCDEFG-243").unwrap(),
+			2,
 			PacketData::DataStartAck,
 		);
 
@@ -305,7 +340,8 @@ mod tests {
 		let packet = Packet::new(
 			Hostname::new("N0CALL-0").unwrap(),
 			Hostname::new("ABCDEFG-243").unwrap(),
-			PacketData::Ping(37),
+			37,
+			PacketData::Ping,
 		);
 
 		let mut buf = vec![];
@@ -329,4 +365,37 @@ mod tests {
 			}
 		}
 	}
+
+	#[test]
+	fn seq_num_invalid_if_eq() {
+		for i in 0..=255 {
+			assert!(!is_valid_seq_num(i, i));
+		}
+	}
+
+	#[test]
+	fn seq_num_valid_if_greater() {
+		for i in 0..=254 {
+			for j in (i + 1)..=255 {
+				assert!(is_valid_seq_num(i, j), "{} -> {} was not valid", i, j);
+			}
+		}
+	}
+
+	#[test]
+	fn seq_num_valid_if_less_but_within_100() {
+		for i in 156i32..=255 {
+			let max = (i + 99) % 255;
+
+			assert!(max < i);
+
+			for j in 0..=max {
+				assert!(is_valid_seq_num(i as u8, j as u8));
+			}
+
+			for j in (max + 1)..=i {
+				assert!(!is_valid_seq_num(i as u8, j as u8));
+			}
+		}
+	}
 }
